<?php

namespace custif\includes;

// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Assets Manager Class
 *
 * Handles registration and enqueuing of all CSS and JavaScript assets
 * required by the Custom IFrame Widget for Elementor.
 *
 * @since 1.0.0
 */
class Assets_Manager {
	/**
	 * Class constructor.
	 *
	 * Sets up all the necessary hooks for enqueueing assets in both
	 * frontend and editor contexts.
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function __construct() {
		add_action( 'wp_enqueue_scripts', array( $this, 'register_frontend_assets' ) );
		add_action( 'elementor/frontend/after_enqueue_styles', array( $this, 'enqueue_frontend_styles' ) );
		add_action( 'elementor/frontend/after_register_scripts', array( $this, 'enqueue_frontend_scripts' ) );
	}

	/**
	 * Register frontend assets.
	 *
	 * Registers all CSS and JavaScript files needed for the frontend.
	 * This includes the main widget styles, and widget scripts.
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function register_frontend_assets() {
		wp_register_style(
			'custif-styles',
			CUSTIF_URL . 'assets/css/style.css',
			array(),
			CUSTIF_VERSION
		);

		wp_register_script(
			'custif-scripts',
			CUSTIF_URL . 'assets/js/widget.js',
			array( 'jquery' ),
			CUSTIF_VERSION,
			true
		);
	}

	/**
	 * Enqueue frontend styles.
	 *
	 * Enqueues the main widget stylesheet for the frontend.
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function enqueue_frontend_styles() {
		wp_enqueue_style( 'custif-styles' );
	}

	/**
	 * Enqueue frontend scripts.
	 *
	 * Enqueues all JavaScript files needed for the widget functionality
	 * including the main widget script.
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function enqueue_frontend_scripts() {
		wp_enqueue_script( 'custif-scripts' );
	}
}

new Assets_Manager();
