<?php

namespace custif\includes;

// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Notices Manager Class
 *
 * Handles admin notices for plugin dependencies and pro features.
 *
 * @since 1.0.0
 */
class Notice_Manager {
	/**
	 * Class constructor.
	 *
	 * Initializes the admin notice functionality by hooking into WordPress admin_init.
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function __construct() {
		add_action( 'admin_init', array( $this, 'check_elementor' ) );
	}

	/**
	 * Check if Elementor is installed and activated.
	 *
	 * Adds an admin notice if Elementor is not active.
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function check_elementor() {
		if ( ! did_action( 'elementor/loaded' ) ) {
			add_action( 'admin_notices', array( $this, 'elementor_missing_notice' ) );
		}
	}

	/**
	 * Display admin notice for missing Elementor dependency.
	 *
	 * Shows a notice with appropriate button to either install or activate Elementor
	 * based on its current installation status.
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function elementor_missing_notice() {
		if ( ! current_user_can( 'install_plugins' ) ) {
			return;
		}

		$elementor_installed = file_exists( WP_PLUGIN_DIR . '/elementor/elementor.php' );

		// translators: Text for the button to activate Elementor plugin.
		$button_text = $elementor_installed ?
			__( 'Activate Elementor Now', 'custom-iframe' ) :
			// translators: Text for the button to install Elementor plugin.
			__( 'Install Elementor Now', 'custom-iframe' );

		$button_url = $elementor_installed ?
			wp_nonce_url(
				admin_url( 'plugins.php?action=activate&plugin=elementor/elementor.php' ),
				'activate-plugin_elementor/elementor.php'
			) :
			wp_nonce_url(
				self_admin_url( 'update.php?action=install-plugin&plugin=elementor' ),
				'install-plugin_elementor'
			);

		printf(
			'<div class="notice notice-error is-dismissible"><p>%s</p><p><a href="%s" class="button-primary">%s</a></p></div>',
			// translators: Notice text shown when Elementor is not installed or activated.
			esc_html__(
				'Custom iFrame Widget requires Elementor to be installed and active.',
				'custom-iframe'
			),
			esc_url( $button_url ),
			esc_html( $button_text )
		);
	}
}

new Notice_Manager();
