<?php

namespace custif\widget;

use Elementor\Controls_Manager;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Widget_Base;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}


/**
 * Custom IFrame Widget Class
 *
 * Custom widget for Elementor that allows embedding iframes with advanced features
 * including lazy loading, auto-height, custom styling, and security controls.
 *
 * @since 1.0.0
 */
class Custom_IFrame_Widget extends Widget_Base {
	/**
	 * Widget constructor.
	 *
	 * Initialize the widget and set up the notices manager.
	 *
	 * @param  array      $data  Widget data.
	 * @param  array|null $args  Widget arguments.
	 *
	 * @since 1.0.0
	 */
	public function __construct( $data = array(), $args = null ) {
		parent::__construct( $data, $args );
	}

	/**
	 * Get widget name.
	 *
	 * @return string Widget name.
	 * @since 1.0.0
	 */
	public function get_name() {
		return 'custif_iframe_widget';
	}

	/**
	 * Get widget title.
	 *
	 * @return string Widget title.
	 * @since 1.0.0
	 */
	public function get_title() {
		return __( 'Custom iFrame', 'custom-iframe' );
	}

	/**
	 * Get widget icon.
	 *
	 * @return string Widget icon.
	 * @since 1.0.0
	 */
	public function get_icon() {
		return 'eicon-code';
	}

	/**
	 * Get widget categories.
	 *
	 * @return array Widget categories.
	 * @since 1.0.0
	 */
	public function get_categories() {
		return array( 'general' );
	}

	/**
	 * Get widget script dependencies.
	 *
	 * @return array Widget script dependencies.
	 * @since 1.0.0
	 */
	public function get_script_depends() {
		return array( 'custif-scripts' );
	}

	/**
	 * Register widget controls.
	 *
	 * Register all the widget controls/settings.
	 *
	 * @return void
	 * @since 1.0.0
	 */
	protected function register_controls() {

		$this->start_controls_section(
			'content_section',
			array(
				'label' => __( 'Content', 'custom-iframe' ),
				'tab'   => Controls_Manager::TAB_CONTENT,
			)
		);

		$this->add_control(
			'iframe_url',
			array(
				'label'         => esc_html__( 'Source URL', 'custom-iframe' ),
				'type'          => Controls_Manager::URL,
				'dynamic'       => array( 'active' => true ),
				'default'       => array( 'url' => 'https://example.com' ),
				'options'       => false,
				'placeholder'   => esc_html__( 'https://example.com', 'custom-iframe' ),
				'label_block'   => true,
				'show_external' => false,
			)
		);

		$this->add_control(
			'custif_custom_id',
			array(
				'label'       => __( 'Custom ID', 'custom-iframe' ),
				'type'        => Controls_Manager::TEXT,
				'placeholder' => __( 'my-custom-iframe', 'custom-iframe' ),
				'description' => __(
					'Enter an ID for custom CSS or JavaScript. Leave empty for an auto-generated ID.',
					'custom-iframe'
				),
				'label_block' => true,
				'separator'   => 'before',
			)
		);

		$this->add_responsive_control(
			'iframe_height',
			array(
				'label'      => __( 'Height', 'custom-iframe' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => array( 'px', 'vh', '%' ),
				'range'      => array(
					'px' => array(
						'min' => 0,
						'max' => 2000,
					),
					'vh' => array(
						'min' => 0,
						'max' => 100,
					),
					'%'  => array(
						'min' => 0,
						'max' => 100,
					),
				),
				'default'    => array(
					'unit' => 'px',
					'size' => 500,
				),
				'selectors'  => array(
					'{{WRAPPER}} iframe' => 'height: {{SIZE}}{{UNIT}};',
				),
				'condition'  => array(
					'auto_height!' => 'yes',
				),
				'separator'  => 'before',
			)
		);

		$this->add_control(
			'auto_height',
			array(
				'label'        => __( 'Auto Height', 'custom-iframe' ),
				'type'         => Controls_Manager::SWITCHER,
				'label_on'     => __( 'Yes', 'custom-iframe' ),
				'label_off'    => __( 'No', 'custom-iframe' ),
				'return_value' => 'yes',
				'default'      => 'no',
			)
		);

		$this->add_control(
			'show_scrollbars',
			array(
				'label'   => __( 'Show Scrollbars', 'custom-iframe' ),
				'type'    => Controls_Manager::SWITCHER,
				'default' => 'yes',
			)
		);

		$this->add_control(
			'refresh_interval',
			array(
				'label'       => __( 'Refresh Interval (seconds)', 'custom-iframe' ),
				'type'        => Controls_Manager::NUMBER,
				'min'         => 0,
				'step'        => 1,
				'default'     => 0,
				'description' => __( 'Set 0 to disable auto-refresh', 'custom-iframe' ),
			)
		);

		$this->end_controls_section();

		$this->start_controls_section(
			'performance_settings_section',
			array(
				'label' => __( 'Smart Load', 'custom-iframe' ),
				'tab'   => Controls_Manager::TAB_CONTENT,
			)
		);

		$this->add_control(
			'enable_lazy_load',
			array(
				'label'        => __( 'Lazy Load', 'custom-iframe' ),
				'type'         => Controls_Manager::SWITCHER,
				'label_on'     => __( 'Yes', 'custom-iframe' ),
				'label_off'    => __( 'No', 'custom-iframe' ),
				'return_value' => 'yes',
				'default'      => 'no',
				'separator'    => 'before',
			)
		);

		$this->add_control(
			'placeholder_image',
			array(
				'label'     => __( 'Placeholder Image', 'custom-iframe' ),
				'type'      => Controls_Manager::MEDIA,
				'condition' => array(
					'enable_lazy_load' => 'yes',
				),
				'separator' => 'before',
			)
		);

		$this->end_controls_section();

		$this->start_controls_section(
			'style_section',
			array(
				'label' => __( 'Style', 'custom-iframe' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			)
		);

		$this->add_responsive_control(
			'iframe_padding',
			array(
				'label'      => __( 'Padding', 'custom-iframe' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => array( 'px', 'em', '%' ),
				'selectors'  => array(
					'{{WRAPPER}} iframe' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				),
				'separator'  => 'before',
			)
		);

		$this->add_control(
			'align',
			array(
				'label'        => __( 'Alignment', 'custom-iframe' ),
				'type'         => Controls_Manager::CHOOSE,
				'options'      => array(
					'left'   => array(
						'title' => __( 'Left', 'custom-iframe' ),
						'icon'  => 'eicon-text-align-left',
					),
					'center' => array(
						'title' => __( 'Center', 'custom-iframe' ),
						'icon'  => 'eicon-text-align-center',
					),
					'right'  => array(
						'title' => __( 'Right', 'custom-iframe' ),
						'icon'  => 'eicon-text-align-right',
					),
				),
				'prefix_class' => 'custif-iframe-align-',
				'separator'    => 'before',
			)
		);

		$this->add_responsive_control(
			'iframe_container_width',
			array(
				'label'      => __( 'Container Width', 'custom-iframe' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => array( 'px', '%', 'vw' ),
				'range'      => array(
					'px' => array(
						'min' => 180,
						'max' => 1200,
					),
					'%'  => array(
						'min' => 10,
						'max' => 100,
					),
					'vw' => array(
						'min' => 10,
						'max' => 100,
					),
				),
				'default'    => array(
					'unit' => '%',
					'size' => 100,
				),
				'selectors'  => array(
					'{{WRAPPER}} .custif-iframe-wrapper' => 'max-width: {{SIZE}}{{UNIT}};',
				),
				'separator'  => 'before',
			)
		);

		$this->end_controls_section();

		$this->start_controls_section(
			'background_style',
			array(
				'label' => __( 'Background Style', 'custom-iframe' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			)
		);
		$this->add_group_control(
			Group_Control_Background::get_type(),
			array(
				'name'      => 'iframe_background',
				'label'     => __( 'Background Style', 'custom-iframe' ),
				'types'     => array( 'classic', 'gradient' ),
				'selector'  => '{{WRAPPER}} iframe',
				'separator' => 'before',
			)
		);
		$this->end_controls_section();

		$this->start_controls_section(
			'border_style',
			array(
				'label' => __( 'Border Style', 'custom-iframe' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			)
		);
		$this->add_group_control(
			Group_Control_Border::get_type(),
			array(
				'name'     => 'iframe_border',
				'selector' => '{{WRAPPER}} iframe',
			)
		);

		$this->add_control(
			'iframe_border_radius',
			array(
				'label'      => __( 'Border Radius', 'custom-iframe' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => array( 'px', '%' ),
				'selectors'  => array(
					'{{WRAPPER}} iframe' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				),
			)
		);
		$this->end_controls_section();

		$this->start_controls_section(
			'box_style',
			array(
				'label' => __( 'Box Shadow Style', 'custom-iframe' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			)
		);
		$this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			array(
				'name'     => 'iframe_box_shadow',
				'selector' => '{{WRAPPER}} iframe',
			)
		);
		$this->end_controls_section();

		if ( defined( 'CUSTIF_PATH' ) ) {
			include CUSTIF_PATH . '/widget/class-need-help-controller.php';
		}
	}

	/**
	 * Convert YouTube URL into an embeddable URL.
	 *
	 * This function converts social links from different formats
	 *
	 * @param  string $url  The original social media post URL.
	 *
	 * @return string The converted embed URL or the original URL if invalid.
	 * @since 1.0.0
	 */
	public function convert_social_to_embed( $url ) {
		$parsed_url = wp_parse_url( $url );

		if ( ! isset( $parsed_url['host'] ) ) {
			return $url; // Return original if invalid.
		}

		$host       = str_replace( 'www.', '', $parsed_url['host'] );
		$path       = isset( $parsed_url['path'] ) ? trim( $parsed_url['path'], '/' ) : '';
		$googlepath = isset( $parsed_url['path'] ) ? $parsed_url['path'] : '';
		$query      = $parsed_url['query'] ?? '';
		$video_id   = '';

		// YouTube Embed.
		if ( 'youtu.be' === $host ) {
			$video_id = $path;

			return "https://www.youtube.com/embed/$video_id";
		} elseif ( 'youtube.com' === $host ) {
			parse_str( $query, $query_params );
			$video_id = $query_params['v'] ?? '';

			return $video_id ? "https://www.youtube.com/embed/$video_id" : $url;
		}

		// Instagram Embed (Reels & Posts).
		if ( 'instagram.com' === $host ) {
			$parts = explode( '/', $path );
			if ( count( $parts ) >= 2 && in_array( $parts[0], array( 'p', 'reel', 'tv' ) ) ) {
				return "https://www.instagram.com/{$parts[0]}/{$parts[1]}/embed";
			}
		}

		// Figma Embed.
		if ( 'figma.com' === $host ) {
			return 'https://www.figma.com/embed?embed_host=figma&url=' . urlencode( $url );
		}

		// Spotify Embed.
		if ( 'open.spotify.com' === $host ) {
			$parts = explode( '/', $path );

			if ( str_contains( $path, 'embed' ) ) {
				return $url;
			}

			if ( count( $parts ) >= 2 ) {
				return "https://open.spotify.com/embed/{$parts[0]}/{$parts[1]}";
			}
		}

		// SoundCloud Embed.
		if ( 'soundcloud.com' === $host ) {
			return 'https://w.soundcloud.com/player/?url=' . urlencode( $url );
		}

		// vimeo embed.
		if ( 'vimeo.com' === $host ) {
			if ( str_contains( $path, 'player' ) ) {
				return $url;
			}

			$parts = explode( '/', $path );
			if ( count( $parts ) >= 1 && is_numeric( $parts[0] ) ) {
				return "https://player.vimeo.com/video/{$parts[0]}";
			}
		}

		// X embed.
		if ( 'twitter.com' === $host || 'x.com' === $host ) {
			$path_segments = explode( '/', trim( $path, '/' ) );
			$tweet_id      = end( $path_segments );

			if ( is_numeric( $tweet_id ) ) {
				$tweet_url     = 'https://twitter.com/' . $path_segments[0] . '/status/' . $tweet_id;
				$oembed_params = array(
					'url' => $tweet_url,
				);
				$oembed_url    = add_query_arg( $oembed_params, 'https://publish.twitter.com/oembed' );
				$response      = wp_remote_get( $oembed_url );

				if ( is_wp_error( $response ) ) {
					return '<p>Error fetching tweet. Please try again.</p>';
				}

				$body = wp_remote_retrieve_body( $response );
				$data = json_decode( $body, true );

				// Return embedded Tweet HTML if available.
				if ( isset( $data['html'] ) ) {
					return $data['html'];
				} else {
					return '<div class="custif-iframe-notice"><p>' . esc_html__(
						'Invalid response.',
						'custom-iframe'
					) . '</p></div>';
				}
			}
		}

		// Embed URL for Google Docs, Sheets, Slides, and Forms.
		if ( 'docs.google.com' === $host ) {
			if ( preg_match( '/\/document\//', $googlepath ) ) {
				if ( ! str_contains( $url, '?embedded=true' ) ) {
					$embed_url = str_replace( '/pub', '/pub?embedded=true', $url );
				} else {
					$embed_url = $url;
				}
			} elseif ( preg_match( '/\/spreadsheets\//', $googlepath ) ) {
				$embed_url = str_replace( '/edit', '/pubhtml', $url );
			} elseif ( preg_match( '/\/presentation\//', $googlepath ) ) {
				$embed_url = str_replace( '/pub', '/embed', $url );
			} elseif ( preg_match( '/\/forms\//', $googlepath ) ) {
				if ( str_contains( $url, '?usp=header' ) ) {
					$embed_url = str_replace( '?usp=header', '?embedded=true', $url );
				} else {
					$embed_url = $url;
				}
			} else {
				return '<div class="custif-iframe-notice"><p>' . esc_html__(
					'Unsupported Google App URL.',
					'custom-iframe'
				) . '</p></div>';
			}

			return esc_url( $embed_url );
		}

		return $url; // Return original if no match.
	}


	/**
	 * Render widget output.
	 *
	 * Written in PHP and used to generate the final HTML.
	 *
	 * @return void
	 * @since 1.0.0
	 */
	protected function render() {
		$settings  = $this->get_settings_for_display();
		$iframe_id = ! empty( $settings['custif_custom_id'] ) ? esc_attr( $settings['custif_custom_id'] ) : 'custif-iframe-' . $this->get_id();

		// Properly get URL from settings.
		$url = ! empty( $settings['iframe_url']['url'] ) ? $settings['iframe_url']['url'] : '';
		$url = $this->convert_social_to_embed( $url );

		// Prepare placeholder image if needed.
		$placeholder_style = '';
		if ( ! empty( $settings['enable_lazy_load'] ) && ! empty( $settings['placeholder_image']['url'] ) ) {
			$placeholder_style = 'background-image: url(' . esc_url( $settings['placeholder_image']['url'] ) . '); background-size: cover; background-position: center;';
		}

		?>
		<div class="custif-iframe-wrapper" id="<?php echo esc_attr( $iframe_id ); ?>">
			<?php if ( ! empty( $url ) ) : ?>
				<?php if ( filter_var( $url, FILTER_VALIDATE_URL ) ) : ?>
					<iframe
							style="<?php echo esc_attr( $placeholder_style ); ?>"
							scrolling="<?php echo esc_attr( $settings['show_scrollbars'] ? 'yes' : 'no' ); ?>"
							data-auto-height="<?php echo esc_attr( $settings['auto_height'] ); ?>"
							data-refresh-interval="<?php echo esc_attr( $settings['refresh_interval'] ); ?>"
							loading="<?php echo esc_attr( ! empty( $settings['enable_lazy_load'] ) ? 'lazy' : '' ); ?>"
							src="<?php echo esc_url( $url ); ?>"
					></iframe>
				<?php else : ?>
					<?php echo $url; ?>
				<?php endif; ?>
			<?php else : ?>
				<div class="custif-iframe-notice">
					<div class="notice-icon">
						<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" width="24" height="24">
							<path fill="none" stroke="currentColor" stroke-linecap="round" stroke-linejoin="round"
								  stroke-width="2"
								  d="M12 8v4m0 4h.01M22 12c0 5.523-4.477 10-10 10S2 17.523 2 12 6.477 2 12 2s10 4.477 10 10z"/>
						</svg>
					</div>
					<p><?php esc_html_e( 'Please enter a valid URL', 'custom-iframe' ); ?></p>
				</div>
			<?php endif; ?>

		</div>
		<?php
	}
}